#include "../global.h"

using namespace std;

Matrix4::Matrix4()
{
  for (int i=0; i<4; ++i)
  {
    for (int j=0; j<4; ++j)
    {
      m[i][j] = 0;
    }
  }
}
Matrix4::Matrix4(
  double m00, double m01, double m02, double m03,
  double m10, double m11, double m12, double m13,
  double m20, double m21, double m22, double m23,
  double m30, double m31, double m32, double m33 )
{
  m[0][0] = m00;
  m[1][0] = m01;
  m[2][0] = m02;
  m[3][0] = m03;
  m[0][1] = m10;
  m[1][1] = m11;
  m[2][1] = m12;
  m[3][1] = m13;
  m[0][2] = m20;
  m[1][2] = m21;
  m[2][2] = m22;
  m[3][2] = m23;
  m[0][3] = m30;
  m[1][3] = m31;
  m[2][3] = m32;
  m[3][3] = m33;
}

double* Matrix4::getPointer()
{
  return &m[0][0];
}

void Matrix4::identity()
{
  double ident[4][4]={{1,0,0,0},{0,1,0,0},{0,0,1,0},{0,0,0,1}};
  for (int i=0; i<4; ++i)
  {
    for (int j=0; j<4; ++j)
    {
      m[i][j] = ident[i][j];
    }
  }
}

void Matrix4::set(int column, int row, double value)
{
		m[column][row] = value;//--------------------------start fixing here
}

double Matrix4::get(int column, int row)
{
		return m[column][row];
}

Matrix4 Matrix4::multiply(Matrix4 &a)
{
	Matrix4 result;

	for (int i=0; i<4; ++i)
	{
		for (int j=0; j<4; ++j)
		{
			for (int temp=0; temp<4; ++temp)
			{
				result.m[j][i] += m[temp][i]*a.m[j][temp];
			}
		}
	}

	return result;
}

void Matrix4::multiplyInPlace(Matrix4 &a)
{
	Matrix4 result;

	for (int i=0; i<4; ++i)
	{
		for (int j=0; j<4; ++j)
		{
			for (int temp=0; temp<4; ++temp)
			{
				result.m[j][i] += m[temp][i]*a.m[j][temp];
			}
		}
	}


    for (int i=0; i<4; ++i)
	{
		for (int j=0; j<4; ++j)
		{
			for (int temp=0; temp<4; ++temp)
			{
				m[i][j] = result.m[i][j];
			}
		}
	}
}

Vector3 Matrix4::multiplyVector(Vector3 &a)
{
	Vector4 result(a[0], a[1], a[2],1);
	result=multiplyVector(result);
	result.dehomogenize();
	Vector3 finalresult(result[0], result[1], result[2]);
	return finalresult;
}
Vector4 Matrix4::multiplyVectorPromote(Vector3 &a)
{
	Vector4 result(a[0], a[1], a[2],1);
	result=multiplyVector(result);
	//result.dehomogenize();
	//Vector3 finalresult(result[0], result[1], result[2]);
	return result;
}

Vector4 Matrix4::multiplyVector(Vector4 &a)
{
	Vector4 result;
	double arr[4] = {0,0,0,0};

	for (int i=0; i<4; ++i)
	{
		for (int temp=0; temp<4; ++temp)
		{
			arr[i] += m[temp][i]*a[temp];
		}
	}

	result.set(arr[0], arr[1], arr[2], arr[3]);
	return result;
}

void Matrix4::rotateX(double angle)
{
	double th = BasicMath::radian(angle);
	m[0][0] = 1;
	m[0][1] = 0;
	m[0][2] = 0;
	m[0][3] = 0;
	m[1][0] = 0;
	m[1][1] = cos(th);
	m[1][2] = sin(th);
	m[1][3] = 0;
	m[2][0] = 0;
	m[2][1] = -sin(th);
	m[2][2] = cos(th);
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;
}


// angle in degrees
void Matrix4::rotateY(double angle)
{
	double th = BasicMath::radian(angle);
	m[0][0] = cos(th);
	m[0][1] = 0;
	m[0][2] = -sin(th);
	m[0][3] = 0;
	m[1][0] = 0;
	m[1][1] = 1;
	m[1][2] = 0;
	m[1][3] = 0;
	m[2][0] = sin(th);
	m[2][1] = 0;
	m[2][2] = cos(th);
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;
}

void Matrix4::rotateZ(double angle)
{
	double th = BasicMath::radian(angle);
	m[0][0] = cos(th);
	m[0][1] = sin(th);
	m[0][2] = 0;
	m[0][3] = 0;
	m[1][0] = -sin(th);
	m[1][1] = cos(th);
	m[1][2] = 0;
	m[1][3] = 0;
	m[2][0] = 0;
	m[2][1] = 0;
	m[2][2] = 1;
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;
}

void Matrix4::rotate(double angle, Vector3 &a)
{
	a.normalize();
	double th = BasicMath::radian(angle);
	/*m[0][0] = a[0]*a[0]+cos(th)*(1-(a[0]*a[0]));
	m[0][1] = a[0]*a[1]*(1-cos(th))+a[2]*sin(th);
	m[0][2] = a[0]*a[1]*(1-cos(th))-a[1]*sin(th);
	m[0][3] = 0;
	m[1][0] = a[0]*a[1]*(1-cos(th))-a[2]*sin(th);
	m[1][1] = a[1]*a[1]+cos(th)*(1-(a[1]*a[1]));
	m[1][2] = a[1]*a[2]*(1-cos(th))+a[0]*sin(th);
	m[1][3] = 0;
	m[2][0] = a[0]*a[2]*(1-cos(th))+a[1]*sin(th);
	m[2][1] = a[1]*a[2]*(1-cos(th))-a[0]*sin(th);
	m[2][2] = a[2]*a[2]+cos(th)*(1-a[2]*a[2]);
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;*/


    m[0][0] = 1+(1-cos(th))*(a[0]*a[0]-1);
	m[1][0] = -a[2]*sin(th)+(1-cos(th))*(a[0]*a[1]);
	m[2][0] = a[1]*sin(th)+(1-cos(th))*(a[0]*a[2]);
	m[3][0] = 0;
	m[0][1] = a[2]*sin(th)+(1-cos(th))*(a[1]*a[0]);
	m[1][1] = 1+(1-cos(th))*(a[1]*a[1]-1);
	m[2][1] = -a[0]*sin(th)+(1-cos(th))*(a[1]*a[2]);
	m[3][1] = 0;
	m[0][2] = -a[1]*sin(th)+(1-cos(th))*(a[2]*a[0]);
	m[1][2] = a[0]*sin(th)+(1-cos(th))*(a[2]*a[1]);
	m[2][2] = 1+(1-cos(th))*(a[2]*a[2]-1);
	m[3][2] = 0;
	m[0][3] = 0;
	m[1][3] = 0;
	m[2][3] = 0;
	m[3][3] = 1;

    /*
    m[0][0] = 1+(1-cos(th))*(a[0]*a[0]-1);
	m[0][1] = -a[2]*sin(th)+(1-cos(th))*(a[0]*a[1]);
	m[0][2] = a[1]*sin(th)+(1-cos(th))*(a[0]*a[2]);
	m[0][3] = 0;
	m[1][0] = a[2]*sin(th)+(1-cos(th))*(a[1]*a[0]);
	m[1][1] = 1+(1-cos(th))*(a[1]*a[1]-1);
	m[1][2] = -a[0]*sin(th)+(1-cos(th))*(a[1]*a[2]);
	m[1][3] = 0;
	m[2][0] = -a[1]*sin(th)+(1-cos(th))*(a[2]*a[0]);
	m[2][1] = a[0]*sin(th)+(1-cos(th))*(a[2]*a[1]);
	m[2][2] = 1+(1-cos(th))*(a[2]*a[2]-1);
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;*/
}

void Matrix4::rotate(double angle, Vector4 &a)
{
	Vector3 v(a[0],a[1],a[2]);
	rotate(angle,v);
}

void Matrix4::scaling(double x0, double y0, double z0)
{
	m[0][0] = x0;
	m[0][1] = 0;
	m[0][2] = 0;
    m[0][3] = 0;
    m[1][0] = 0;
	m[1][1] = y0;
	m[1][2] = 0;
	m[1][3] = 0;
	m[2][0] = 0;
	m[2][1] = 0;
	m[2][2] = z0;
	m[2][3] = 0;
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;
}

void Matrix4::translation(double x0, double y0, double z0)
{
	m[0][0] = 1;
	m[0][1] = 0;
	m[0][2] = 0;
	m[0][3] = 0;
	m[1][0] = 0;
	m[1][1] = 1;
	m[1][2] = 0;
	m[1][3] = 0;
	m[2][0] = 0;
    m[2][1] = 0;
	m[2][2] = 1;
	m[2][3] = 0;
	m[3][1] = y0;
	m[3][2] = z0;
	m[3][0] = x0;
	m[3][3] = 1;
}

void Matrix4::invert(){
	double a0 = m[0][0]*m[1][1] - m[1][0]*m[0][1];
	double a1 = m[0][0]*m[2][1] - m[2][0]*m[0][1];
	double a2 = m[0][0]*m[3][1] - m[3][0]*m[0][1];
	double a3 = m[1][0]*m[2][1] - m[2][0]*m[1][1];
	double a4 = m[1][0]*m[3][1] - m[3][0]*m[1][1];
	double a5 = m[2][0]*m[3][1] - m[3][0]*m[2][1];
	double b0 = m[0][2]*m[1][3] - m[1][2]*m[0][3];
	double b1 = m[0][2]*m[2][3] - m[2][2]*m[0][3];
	double b2 = m[0][2]*m[3][3] - m[3][2]*m[0][3];
	double b3 = m[1][2]*m[2][3] - m[2][2]*m[1][3];
	double b4 = m[1][2]*m[3][3] - m[3][2]*m[1][3];
	double b5 = m[2][2]*m[3][3] - m[3][2]*m[2][3];

	double det = a0*b5 - a1*b4 + a2*b3 + a3*b2 - a4*b1 + a5*b0;

	double inv[4][4];
	inv[0][0] = + m[1][1]*b5 - m[2][1]*b4 + m[3][1]*b3;
	inv[0][1] = - m[0][1]*b5 + m[2][1]*b2 - m[3][1]*b1;
	inv[0][2] = + m[0][1]*b4 - m[1][1]*b2 + m[3][1]*b0;
	inv[0][3] = - m[0][1]*b3 + m[1][1]*b1 - m[2][1]*b0;
	inv[1][0] = - m[1][0]*b5 + m[2][0]*b4 - m[3][0]*b3;
	inv[1][1] = + m[0][0]*b5 - m[2][0]*b2 + m[3][0]*b1;
	inv[1][2] = - m[0][0]*b4 + m[1][0]*b2 - m[3][0]*b0;
	inv[1][3] = + m[0][0]*b3 - m[1][0]*b1 + m[2][0]*b0;
	inv[2][0] = + m[1][3]*a5 - m[2][3]*a4 + m[3][3]*a3;
	inv[2][1] = - m[0][3]*a5 + m[2][3]*a2 - m[3][3]*a1;
	inv[2][2] = + m[0][3]*a4 - m[1][3]*a2 + m[3][3]*a0;
	inv[2][3] = - m[0][3]*a3 + m[1][3]*a1 - m[2][3]*a0;
	inv[3][0] = - m[1][2]*a5 + m[2][2]*a4 - m[3][2]*a3;
	inv[3][1] = + m[0][2]*a5 - m[2][2]*a2 + m[3][2]*a1;
	inv[3][2] = - m[0][2]*a4 + m[1][2]*a2 - m[3][2]*a0;
	inv[3][3] = + m[0][2]*a3 - m[1][2]*a1 + m[2][2]*a0;

	double invDet = 1/det;
	//cout << det << endl;
	inv[0][0] *= invDet;
	inv[1][0] *= invDet;
	inv[2][0] *= invDet;
	inv[3][0] *= invDet;
	inv[0][1] *= invDet;
	inv[1][1] *= invDet;
	inv[2][1] *= invDet;
	inv[3][1] *= invDet;
	inv[0][2] *= invDet;
	inv[1][2] *= invDet;
	inv[2][2] *= invDet;
	inv[3][2] *= invDet;
	inv[0][3] *= invDet;
	inv[1][3] *= invDet;
	inv[2][3] *= invDet;
	inv[3][3] *= invDet;

	m[0][0] = inv[0][0];
	m[1][0] = inv[1][0];
	m[2][0] = inv[2][0];
	m[3][0] = inv[3][0];
	m[0][1] = inv[0][1];
	m[1][1] = inv[1][1];
	m[2][1] = inv[2][1];
	m[3][1] = inv[3][1];
	m[0][2] = inv[0][2];
	m[1][2] = inv[1][2];
	m[2][2] = inv[2][2];
	m[3][2] = inv[3][2];
	m[0][3] = inv[0][3];
	m[1][3] = inv[1][3];
	m[2][3] = inv[2][3];
	m[3][3] = inv[3][3];

}

Matrix4 Matrix4::getInverse(){
	double a0 = m[0][0]*m[1][1] - m[1][0]*m[0][1];
	double a1 = m[0][0]*m[2][1] - m[2][0]*m[0][1];
	double a2 = m[0][0]*m[3][1] - m[3][0]*m[0][1];
	double a3 = m[1][0]*m[2][1] - m[2][0]*m[1][1];
	double a4 = m[1][0]*m[3][1] - m[3][0]*m[1][1];
	double a5 = m[2][0]*m[3][1] - m[3][0]*m[2][1];
	double b0 = m[0][2]*m[1][3] - m[1][2]*m[0][3];
	double b1 = m[0][2]*m[2][3] - m[2][2]*m[0][3];
	double b2 = m[0][2]*m[3][3] - m[3][2]*m[0][3];
	double b3 = m[1][2]*m[2][3] - m[2][2]*m[1][3];
	double b4 = m[1][2]*m[3][3] - m[3][2]*m[1][3];
	double b5 = m[2][2]*m[3][3] - m[3][2]*m[2][3];

	double det = a0*b5 - a1*b4 + a2*b3 + a3*b2 - a4*b1 + a5*b0;

	double inv[4][4];
	inv[0][0] = + m[1][1]*b5 - m[2][1]*b4 + m[3][1]*b3;
	inv[0][1] = - m[0][1]*b5 + m[2][1]*b2 - m[3][1]*b1;
	inv[0][2] = + m[0][1]*b4 - m[1][1]*b2 + m[3][1]*b0;
	inv[0][3] = - m[0][1]*b3 + m[1][1]*b1 - m[2][1]*b0;
	inv[1][0] = - m[1][0]*b5 + m[2][0]*b4 - m[3][0]*b3;
	inv[1][1] = + m[0][0]*b5 - m[2][0]*b2 + m[3][0]*b1;
	inv[1][2] = - m[0][0]*b4 + m[1][0]*b2 - m[3][0]*b0;
	inv[1][3] = + m[0][0]*b3 - m[1][0]*b1 + m[2][0]*b0;
	inv[2][0] = + m[1][3]*a5 - m[2][3]*a4 + m[3][3]*a3;
	inv[2][1] = - m[0][3]*a5 + m[2][3]*a2 - m[3][3]*a1;
	inv[2][2] = + m[0][3]*a4 - m[1][3]*a2 + m[3][3]*a0;
	inv[2][3] = - m[0][3]*a3 + m[1][3]*a1 - m[2][3]*a0;
	inv[3][0] = - m[1][2]*a5 + m[2][2]*a4 - m[3][2]*a3;
	inv[3][1] = + m[0][2]*a5 - m[2][2]*a2 + m[3][2]*a1;
	inv[3][2] = - m[0][2]*a4 + m[1][2]*a2 - m[3][2]*a0;
	inv[3][3] = + m[0][2]*a3 - m[1][2]*a1 + m[2][2]*a0;

	double invDet = 1/det;
	//cout << det << endl;
	inv[0][0] *= invDet;
	inv[1][0] *= invDet;
	inv[2][0] *= invDet;
	inv[3][0] *= invDet;
	inv[0][1] *= invDet;
	inv[1][1] *= invDet;
	inv[2][1] *= invDet;
	inv[3][1] *= invDet;
	inv[0][2] *= invDet;
	inv[1][2] *= invDet;
	inv[2][2] *= invDet;
	inv[3][2] *= invDet;
	inv[0][3] *= invDet;
	inv[1][3] *= invDet;
	inv[2][3] *= invDet;
	inv[3][3] *= invDet;

    //return Matrix4(inv[0][0],inv[0][1],inv[0][2],inv[0][3],inv[1][0],inv[1][1],inv[1][2],inv[1][3],inv[2][0],inv[2][1],inv[2][2],inv[2][3],inv[3][0],inv[3][1],inv[3][2],inv[3][3]);
    return Matrix4(inv[0][0],inv[1][0],inv[2][0],inv[3][0],inv[0][1],inv[1][1],inv[2][1],inv[3][1],inv[0][2],inv[1][2],inv[2][2],inv[3][2],inv[0][3],inv[1][3],inv[2][3],inv[3][3]);

}

/*
int main()
{

	Matrix4 m3(1,0,0,0,0,-1,0,0,0,0,1,40,0,0,0,1);
	//Matrix4 m2(17,18,19,20,5,6,7,8,20,19,18,17,16,15,14,13);
	//Matrix4 m3;
	//m3 = m2.multiply(m1);
	m3.invert();
	cout << m3.get(0,0) << ":" <<m3.get(0,1) << ":" <<m3.get(0,2) << ":" <<m3.get(0,3) << ":" << endl;
	cout << m3.get(1,0) << ":" <<m3.get(1,1) << ":" <<m3.get(1,2) << ":" <<m3.get(1,3) << ":" << endl;
	cout << m3.get(2,0) << ":" <<m3.get(2,1) << ":" <<m3.get(2,2) << ":" <<m3.get(2,3) << ":" << endl;
	cout << m3.get(3,0) << ":" <<m3.get(3,1) << ":" <<m3.get(3,2) << ":" <<m3.get(3,3) << ":" << endl;

    //Vector4 v(1,2,3,4);
	//Vector4 o = m1.multiplyVector(v);
	//cout << o.get(0) << ":" << o.get(1) << ":" << o.get(2) << ":"<< o.get(3) << endl;
}*/

