#include "global.h"
using namespace std;


transformGroup* makeStreets(){
    Matrix4 tempTrans;
	Matrix4 tempRot;
    Matrix4 tempScale;

    //create the root
    transformGroup *STREETS = new transformGroup;//stretch it using the matrix we're passing in

	tempTrans.identity();
	tempRot.identity();
	//left-side street, move up ten units
	tempTrans.translation(0, 0, -CORNER_SIZE);
	transformGroup* street = new transformGroup(tempTrans);
    street->addChild(new Street());
    STREETS->addChild(street);


	tempTrans.identity();
	tempRot.identity();
	//top-side street
	//first rotate to the right by 90/deg
	tempRot.rotateY(-90);
	//then translate into -z and 10 to the right
	tempTrans.translation(-BLOCK_SIZE, 0, -CORNER_SIZE);
	
	transformGroup* top_street = new transformGroup(tempRot.multiply(tempTrans));
    top_street->addChild(new Street());
    STREETS->addChild(top_street);


	tempTrans.identity();
	tempRot.identity();
	//bottom-side street
	//first rotate to the right by 90/deg
	tempRot.rotateY(-90);
	//then translate 10 to the right
	tempTrans.translation(-CORNER_SIZE, 0, -CORNER_SIZE);
	
	transformGroup* bot_street = new transformGroup(tempRot.multiply(tempTrans));
    bot_street->addChild(new Street());
    STREETS->addChild(bot_street);

	tempTrans.identity();
	tempRot.identity();
	//right-side street
	//first rotate to the right by 90/deg
	//then translate 10 to the right
	tempTrans.translation(STREET_LENGTH+CORNER_SIZE, 0, -CORNER_SIZE);
	
	transformGroup* right_street = new transformGroup(tempRot.multiply(tempTrans));
    right_street->addChild(new Street());
    STREETS->addChild(right_street);

	return STREETS;

}



transformGroup* makeCorners(){
    Matrix4 tempTrans;
	Matrix4 tempRot;
    Matrix4 tempScale;

    //create the root
    transformGroup *CORNERS = new transformGroup;//stretch it using the matrix we're passing in

	tempTrans.identity();
	tempRot.identity();
	//bottom-left corner
	tempTrans.translation(0, 0, 0);
	transformGroup* street = new transformGroup(tempTrans);
    street->addChild(new Corner());
    CORNERS->addChild(street);


	tempTrans.identity();
	tempRot.identity();
	//top-left corner
	//then translate into -z
	tempTrans.translation(0, 0, -(STREET_LENGTH+CORNER_SIZE));
	
	transformGroup* top_street = new transformGroup(tempRot.multiply(tempTrans));
    top_street->addChild(new Corner());
    CORNERS->addChild(top_street);


	tempTrans.identity();
	tempRot.identity();
	//top-right street
	//translate into z and -90 to the right
	tempTrans.translation((STREET_LENGTH+CORNER_SIZE), 0, -(STREET_LENGTH+CORNER_SIZE));
	
	transformGroup* bot_street = new transformGroup(tempRot.multiply(tempTrans));
    bot_street->addChild(new Corner());
    CORNERS->addChild(bot_street);

	tempTrans.identity();
	tempRot.identity();
	//bottom-right corner
	//then translate 10 to the right
	tempTrans.translation((STREET_LENGTH+CORNER_SIZE), 0, 0);
	
	transformGroup* right_street = new transformGroup(tempRot.multiply(tempTrans));
    right_street->addChild(new Corner());
    CORNERS->addChild(right_street);

	return CORNERS;

}


//this makes max_trees random trees in a grid that is x_width on the x-axis wide and z_length on the -z axis long
//offsets the center of the trees at least TREE_OFFSET from the sides 
transformGroup* makeRndTreeField(int x_width, int z_length, int max_trees){

	int i;
	Matrix4 tempTrans; 	tempTrans.identity();
	Matrix4 tempRot; 	tempRot.identity();
    Matrix4 tempScale; 	tempScale.identity();

    //create the root
    transformGroup *ROOT = new transformGroup;
	transformGroup *tree;
	
	for (i = 0; i < max_trees; i++){
		
		//random scaling
		double scale = (unitRnd() * (MAX_TREE_SCALE - MIN_TREE_SCALE)) + MIN_TREE_SCALE;
		tempScale.scaling(scale,scale,scale);
		
		//random offset
		double x_max = (x_width-TREE_OFFSET), x_min = TREE_OFFSET;
		double z_max = (z_length-TREE_OFFSET), z_min = TREE_OFFSET;
		
		double x_offs = (unitRnd() * (x_max - x_min)) + x_min;
		double z_offs = (unitRnd() * (z_max - z_min)) + z_min;
		tempTrans.translation(x_offs, 0, -z_offs);
		
		//random rotation
		tempRot.rotateY(unitRnd() * 360);
		
		tempTrans.multiplyInPlace(tempScale);
		tempTrans.multiplyInPlace(tempRot);
		
		//make tree
		tree = new transformGroup(tempTrans);
		tree->addChild(makeRandTree());
		
		ROOT->addChild(tree);
		
	}	
	
	
	return ROOT;
}


transformGroup* makeBuildings(){
    Matrix4 tempTrans, tempTrans2, tempTrans3;
	Matrix4 tempRot;
    Matrix4 tempScale;
	int i,j;
	
    //create the root
    transformGroup *BLDGS = new transformGroup;//stretch it using the matrix we're passing in
	transformGroup *bldg;
	
	tempTrans.identity();
	tempRot.identity();
	
	
	transformGroup* row;
	//now make 8x8 Buildings by row into far z
	for (i = 0; i < (FIELD_LENGTH/BUILDING_SIZE); i++){
		tempTrans.identity();
		tempTrans.translation(0, 0, (-BUILDING_SIZE)*i);
		row = new transformGroup(tempTrans);
		
		//in each row make 8 bldgs to the right 
		for (j = 0; j < (FIELD_WIDTH/BUILDING_SIZE); j++){
			
			
			//first we move it to the right spot
			tempTrans2.identity();
			tempTrans2.translation(BUILDING_SIZE*j, 0, 0);
			
			//now if its supposed to be empty: don't create it, make some trees instead
			if (unitRnd() <= EMPTY_PROP){ 
				cout << "making some trees!" << endl;
				
				bldg = new transformGroup(tempTrans2);
				bldg->addChild(makeRndTreeField(BUILDING_SIZE, BUILDING_SIZE, MAX_BLDG_TREES));
				
				
			}else{
			
			
				//now check if we are to scale it down
				if (unitRnd() <= SCALE_PROP){
					double scale_to = (double)((rand()%101)) / 100;
					scale_to = (scale_to * (SCALE_MAX - SCALE_MIN)) + SCALE_MIN;
				
					tempScale.identity();
					tempScale.scaling(scale_to, scale_to, scale_to);
				
					//and move it a little away from the lower left corner
					double move_to_x = (double)((rand()%101)) / 100;		//[0..1]
					double move_to_z = (double)((rand()%101)) / 100;		//[0..1]
					double max_move = (1 - scale_to) * BUILDING_SIZE;		//in actual world units
				
					move_to_x = move_to_x * max_move;				//scale the x move to max we can move this bldg
					move_to_z = move_to_z * max_move;				//scale the x move to max we can move this bldg
				
					tempTrans3.identity();
					tempTrans3.translation(move_to_x, 0, -move_to_z);
				
					//now multiply
					tempTrans2.multiplyInPlace(tempScale);
					tempTrans2.multiplyInPlace(tempTrans3);
			
				}
				
				bldg = new transformGroup(tempTrans2);
				bldg->addChild(makeRandBuilding());
				
			}
			
			
			row->addChild(bldg);
		}
		
		BLDGS->addChild(row);
		
	}
	
	
	return BLDGS;

}



transformGroup* makeCityBlock(){

    Matrix4 tempTrans;
	Matrix4 tempRot;
    Matrix4 tempScale;

    //create the root
    transformGroup *ROOT = new transformGroup;
    
	//add streets
	ROOT->addChild(makeStreets());
	//add corners
	ROOT->addChild(makeCorners());

	//now make the inner field
	tempTrans.identity();
	tempRot.identity();
	//center field
	//translate 10 to the right, and 10 into -z
	tempTrans.translation(CORNER_SIZE, 0, -CORNER_SIZE);
	
	transformGroup* center_field = new transformGroup(tempRot.multiply(tempTrans));
    center_field->addChild(new Field());

//make buildings children of the center field
	center_field->addChild(makeBuildings());

    ROOT->addChild(center_field);

	return ROOT;


}


transformGroup* makeCityGrid(int nBlocks){

    //create the root
    transformGroup *ROOT = new transformGroup;
   

    Matrix4 tempTrans, tempTrans2, tempTrans3;
	Matrix4 tempRot;
    Matrix4 tempScale;
	int i,j, scaled, empty;
	
    transformGroup *block, *temp;
	
	tempTrans.identity();
	tempRot.identity();
	
	
	transformGroup* row;
	//now make nxn Blocks by row into far z
	for (i = 0; i < nBlocks; i++){
		tempTrans.identity();
		tempTrans.translation(0, 0, (-BLOCK_SIZE)*i);
		row = new transformGroup(tempTrans);
		
		//in each row make n blocks to the right 
		for (j = 0; j < nBlocks; j++){
			
			//first we move it to the right spot
			tempTrans2.identity();
			tempTrans2.translation(BLOCK_SIZE*j, 0, 0);
			
			block = new transformGroup(tempTrans2);
			block->addChild(makeCityBlock());
			row->addChild(block);
		}
	
		ROOT->addChild(row);
	}

	/*
	//add a tree at 0,0
	tempScale.identity();
	tempScale.translation(0,0,0);
	temp = new transformGroup(tempScale);
	temp->addChild(makeRandTree());
	
	ROOT->addChild(temp);
	*/
	return ROOT;


}