#include "../global.h"

using namespace std;
extern GLuint *bldg_tex;
extern bool collisionDetected;
extern bool buildings;
//gains for each building texture's coordinates, necc. because they can have different relative sizes
//double tex_coord_gain[N_BLDG_TEX] = {1, 1, 1, 0.5, 0.2, 1, 1, 0.3, 0.5, 1, 1};
double tex_coord_gain[N_BLDG_TEX] = {1, 1, 1, 0.5, 0.2};

#define MIN_HEIGHT				10
#define MAX_HEIGHT				100
#define STEP					10
#define TEX_COORD_MULTIPLIER	0.25


//this makes a random building and returns it

Building* makeRandBuilding(){

	int h = ((rand()%10) + 1) * 10;
	int ntex = rand()%N_BLDG_TEX;
	//int ntex = 10;

	Building* b = new Building(h, ntex);

	return b;
}

Building::Building(){
    transM.identity();
	height = DEF_HEIGHT;
	centerX = (0+BUILDING_SIZE)/2.0;
	centerZ = (0-BUILDING_SIZE)/2.0;
}
Building::Building(int h, int i){
    transM.identity();
	height = h;
	tex_idx = i;
	centerX = (0+BUILDING_SIZE)/2.0;
	centerZ = (0-BUILDING_SIZE)/2.0;
}

Building::Building(Matrix4 m){
    transM = m;
	height = DEF_HEIGHT;
	centerX = (0+BUILDING_SIZE)/2.0;
	centerZ = (0-BUILDING_SIZE)/2.0;
}

Building::Building(int h, Matrix4 m){
    transM = m;
	height = h;
	centerX = (0+BUILDING_SIZE)/2.0;
	centerZ = (0-BUILDING_SIZE)/2.0;
}

void Building::draw(Matrix4 m){

    if(!buildings){
        return;
    }
	int tex_coord = height * TEX_COORD_MULTIPLIER * tex_coord_gain[tex_idx];

   	glPushMatrix();

	glMatrixMode(GL_MODELVIEW);
	//now set matrix
	m.multiplyInPlace(transM);
	glMultMatrixd(m.getPointer());

	 glPushAttrib(GL_ALL_ATTRIB_BITS); // Pushes attributes like current color to attribute stack.

	//draw five sides of a quadratic building, with lower left corner on 0,0 up to height

	//front side
	if (TEXTURING){
		glBindTexture(GL_TEXTURE_2D, bldg_tex[tex_idx]);
		glColor3f(1, 1, 1);
	}else
		glColor3f(0.5, 0.5, 0.5);

	 glBegin(GL_QUADS);


	//lower left corner
	glNormal3f(0, 0, 1);
	glTexCoord2f(0,0);
     glVertex3f(0, 0, 0);

	//upper left corner
     glTexCoord2f(tex_coord,0);
	glVertex3f(0, height, 0);

	//upper right corner
    glTexCoord2f(tex_coord,1);
	 glVertex3f(BUILDING_SIZE, height, 0);

	//lower right corner
     glTexCoord2f(0,1);
	glVertex3f(BUILDING_SIZE, 0, 0);

	 glEnd();

	//left side
	glBegin(GL_QUADS);
		glNormal3f(-1, 0, 0);
		//lower left corner
		glTexCoord2f(0,0);
	    glVertex3f(0, 0, -BUILDING_SIZE);
		//upper left corner
	    glTexCoord2f(tex_coord,0);
		glVertex3f(0, height, -BUILDING_SIZE);
		//upper right corner
	    glTexCoord2f(tex_coord,1);
		glVertex3f(0, height, 0);
		//lower right corner
	     glTexCoord2f(0,1);
		glVertex3f(0, 0, 0);
	glEnd();


	//right side
	glBegin(GL_QUADS);
		glNormal3f(1, 0, 0);
		//lower left corner
		glTexCoord2f(0,0);
	    glVertex3f(BUILDING_SIZE, 0, 0);
		//upper left corner
	    glTexCoord2f(tex_coord,0);
		glVertex3f(BUILDING_SIZE, height, 0);
		//upper right corner
	    glTexCoord2f(tex_coord,1);
		glVertex3f(BUILDING_SIZE, height, -BUILDING_SIZE);
		//lower right corner
	     glTexCoord2f(0,1);
		glVertex3f(BUILDING_SIZE, 0, -BUILDING_SIZE);
	glEnd();

	//back side
	glBegin(GL_QUADS);
		glNormal3f(0, 0, -1);
		//lower left corner, seen from that side, which means is the lower right from the front
		glTexCoord2f(0,0);
	    glVertex3f(BUILDING_SIZE, 0, -BUILDING_SIZE);
		//upper left corner
	    glTexCoord2f(tex_coord,0);
		glVertex3f(BUILDING_SIZE, height, -BUILDING_SIZE);
		//upper right corner
	    glTexCoord2f(tex_coord,1);
		glVertex3f(0, height, -BUILDING_SIZE);
		//lower right corner
	     glTexCoord2f(0,1);
		glVertex3f(0, 0, -BUILDING_SIZE);
	glEnd();


	//top
	glBegin(GL_QUADS);
		//black
    	glColor3f(0, 0, 0);
		//disable texture
		glBindTexture(GL_TEXTURE_2D,0);

		glNormal3f(0, 1, 0);
		//lower left corner, as seen from top, with camera top pointing toward -z
		glTexCoord2f(0,0);
	    glVertex3f(0, height, 0);
		//upper left corner
	    glTexCoord2f(1,0);
		glVertex3f(0, height, -BUILDING_SIZE);
		//upper right corner
	    glTexCoord2f(1,1);
		glVertex3f(BUILDING_SIZE, height, -BUILDING_SIZE);
		//lower right corner
	     glTexCoord2f(0,1);
		glVertex3f(BUILDING_SIZE, height, 0);
	glEnd();



	 glPopAttrib();
	glPopMatrix();


}

void Building::detectCollision(Node* in,Matrix4 inTransforms, Matrix4 m){
    if(!buildings){
        return;
    }
	m.multiplyInPlace(transM);
    Shape3D* inShape = (Shape3D*)in;
    Matrix4 inTF = inTransforms.multiply(inShape->transM);


    Vector3 centerThis(centerX,0,centerZ);
    centerThis = m.multiplyVector(centerThis);
    Vector3 pointIn(inShape->lobj->vertices[inShape->lobj->indices[0]*3],inShape->lobj->vertices[inShape->lobj->indices[0]*3+1],inShape->lobj->vertices[inShape->lobj->indices[0]*3+2]);
    pointIn = inTF.multiplyVector(pointIn);
    //20 is arbitrarily picked
    if(sqrt((centerThis.get(0)-pointIn.get(0))*(centerThis.get(0)-pointIn.get(0))+(centerThis.get(2)-pointIn.get(2))*(centerThis.get(2)-pointIn.get(2)))>BUILDING_SIZE*2){
        cerr<<"skip, it's far away"<<endl;
        return;
    }

    //Matrix4 inTF = inShape->transM.multiply(inTransforms);
    //    for (int i=500; i<501; i++)
    for (int i=0; i<(inShape->lobj->nIndices)-1; i++)
    {
        //calculate line segment
        Vector3 P0(inShape->lobj->vertices[inShape->lobj->indices[i]*3],inShape->lobj->vertices[inShape->lobj->indices[i]*3+1],inShape->lobj->vertices[inShape->lobj->indices[i]*3+2]);
        Vector3 P1(inShape->lobj->vertices[inShape->lobj->indices[i+1]*3],inShape->lobj->vertices[inShape->lobj->indices[i+1]*3+1],inShape->lobj->vertices[inShape->lobj->indices[i+1]*3+2]);
        P0 = inTF.multiplyVector(P0);
        P1 = inTF.multiplyVector(P1);
        P1.set(P1.get(0),P1.get(1),P1.get(2));
        //translate P0 and P1

        /*FIRST SIDE*/
        Vector3 PP0(0, 0, 0);
        Vector3 PP1(0,height,0);
        Vector3 PP2(BUILDING_SIZE,height,0);
        Vector3 PP3(BUILDING_SIZE,0,0);
        Vector3 PNormal(0,0,1);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            return;
        }
        else{
/*
            cerr <<"P0:"<< P0.get(0)<<":"<<P0.get(1)<<":"<<P0.get(2)<< endl;
            cerr <<"P1:"<< P1.get(0)<<":"<<P1.get(1)<<":"<<P1.get(2)<< endl;

            cerr <<"PP0:"<< PP0.get(0)<<":"<<PP0.get(1)<<":"<<PP0.get(2)<< endl;
            cerr <<"PP1:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
            cerr <<"PP2:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
            cerr <<"PP3:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
            cerr <<"PNormal:"<< PNormal.get(0)<<":"<<PNormal.get(1)<<":"<<PNormal.get(2)<< endl;
            */

        }

        /* SECOND SIDE */
        PP0.set(0, 0, -BUILDING_SIZE);
        PP1.set(0, height, -BUILDING_SIZE);
        PP2.set(0,height,0);
        PP3.set(0,0,0);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);

        PNormal.set(-1,0,0);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }

        /* THIRD SIDE */
        PP0.set(BUILDING_SIZE,0,0);
        PP1.set(BUILDING_SIZE, height, 0);
        PP2.set(BUILDING_SIZE, height, -BUILDING_SIZE);
		PP3.set(BUILDING_SIZE, 0, -BUILDING_SIZE);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        PNormal.set(1,0,0);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }

        /* FOURTH SIDE */
        PP0.set(BUILDING_SIZE,0,-BUILDING_SIZE);
        PP1.set(BUILDING_SIZE, height, -BUILDING_SIZE);
        PP2.set(0, height, -BUILDING_SIZE);
        PP3.set(0, 0, -BUILDING_SIZE);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        PNormal.set(0,0,-1);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }
    }
}
