#include "../global.h"


extern bool collisionDetected;
extern bool trees;
using namespace std;


//this makes a random tree and returns it

transformGroup* makeRandTree(){

	//cout << "makeRndTree called!" << endl;
	Matrix4 tempScale;
	tempScale.identity();
	GLuint d;
	float f[8] = {0.353,0.176,0,.25, .0314,.314,.05,1};
	//float f[8] = {0,0,0,1, 0,0,0,1};
	d = compileTree(150, 4, f);

	//now make a tree
	Tree* t = new Tree(d);
	tempScale.scaling(TREE_SCALE,TREE_SCALE,TREE_SCALE);

	transformGroup* tree = new transformGroup(tempScale);
	tree->addChild(t);

	//cout << "makeRndTree returning: " << tree << endl;
	return tree;
}



Tree::Tree(GLuint d){
    transM.identity();
	DList = d;
}

Tree::Tree(Matrix4 m){
    transM = m;
}

void Tree::draw(Matrix4 m){
    if(!trees){
        return;
    }
	//cout << "Tree.draw called!" << endl;

   	glPushMatrix();

	glMatrixMode(GL_MODELVIEW);
	//now set matrix
	m.multiplyInPlace(transM);
	Matrix4 scalem;
	scalem.scaling(1,1.2,1);
	m.multiplyInPlace(scalem);
	glMultMatrixd(m.getPointer());


	//just a draw a single flat quadratic shape, assigning textture coords as well
	//into the z-axis, on x-z plane
	 glPushAttrib(GL_ALL_ATTRIB_BITS);	// Pushes attributes like current color to attribute stack.


	//draw tree
	glCallList(DList);

	 glPopAttrib();
	 glPopMatrix();

}
void Tree::detectCollision(Node* in,Matrix4 inTransforms, Matrix4 m){
    if(!trees){
        return;
    }
	m.multiplyInPlace(transM);
    Shape3D* inShape = (Shape3D*)in;
    Matrix4 inTF = inTransforms.multiply(inShape->transM);



    Vector3 centerThis(0,0,0);
    centerThis = m.multiplyVector(centerThis);
    Vector3 pointIn(inShape->lobj->vertices[inShape->lobj->indices[0]*3],inShape->lobj->vertices[inShape->lobj->indices[0]*3+1],inShape->lobj->vertices[inShape->lobj->indices[0]*3+2]);
    pointIn = inTF.multiplyVector(pointIn);
    //20 is arbitrarily picked
    if(sqrt((centerThis.get(0)-pointIn.get(0))*(centerThis.get(0)-pointIn.get(0))+(centerThis.get(2)-pointIn.get(2))*(centerThis.get(2)-pointIn.get(2)))>5){
        //cerr<<"skip, it's far away"<<endl;
        return;
    }

    //Matrix4 inTF = inShape->transM.multiply(inTransforms);
    //    for (int i=500; i<501; i++)
    for (int i=0; i<(inShape->lobj->nIndices)-1; i++)
    {
        //calculate line segment
        Vector3 P0(inShape->lobj->vertices[inShape->lobj->indices[i]*3],inShape->lobj->vertices[inShape->lobj->indices[i]*3+1],inShape->lobj->vertices[inShape->lobj->indices[i]*3+2]);
        Vector3 P1(inShape->lobj->vertices[inShape->lobj->indices[i+1]*3],inShape->lobj->vertices[inShape->lobj->indices[i+1]*3+1],inShape->lobj->vertices[inShape->lobj->indices[i+1]*3+2]);
        P0 = inTF.multiplyVector(P0);
        P1 = inTF.multiplyVector(P1);
        P1.set(P1.get(0),P1.get(1),P1.get(2));
        //translate P0 and P1

        // Front
        Vector3 PP0(-1, 0, 1);
        Vector3 PP1(-1,10,1);
        Vector3 PP2(1,10,1);
        Vector3 PP3(1,0,1);
        Vector3 PNormal(0,0,1);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            return;
        }
        else{

//            cerr <<"P0:"<< P0.get(0)<<":"<<P0.get(1)<<":"<<P0.get(2)<< endl;
//            cerr <<"P1:"<< P1.get(0)<<":"<<P1.get(1)<<":"<<P1.get(2)<< endl;

//            cerr <<"PP0:"<< PP0.get(0)<<":"<<PP0.get(1)<<":"<<PP0.get(2)<< endl;
//            cerr <<"PP1:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
//            cerr <<"PP2:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
//            cerr <<"PP3:"<< PP1.get(0)<<":"<<PP1.get(1)<<":"<<PP1.get(2)<< endl;
//            cerr <<"PNormal:"<< PNormal.get(0)<<":"<<PNormal.get(1)<<":"<<PNormal.get(2)<< endl;


        }

        // Right SIDE
        PP0.set(1, 0, -1);
        PP1.set(1,10,-1);
        PP2.set(1,10,1);
        PP3.set(1,0,1);
        //Vector3 PNormal(0,0,1);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);

        PNormal.set(1,0,0);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }

        // left SIDE
        PP0.set(-1, 0, -1);
        PP1.set(-1,10,-1);
        PP2.set(-1,10,1);
        PP3.set(-1,0,1);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        PNormal.set(-1,0,0);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }

        // back SIDE
        PP0.set(-1, 0, -1);
        PP1.set(-1,10,-1);
        PP2.set(1,10,-1);
        PP3.set(1,0,-1);
        //PNormal.set(0,0,-1);

        PP0 = m.multiplyVector(PP0);
        PP1 = m.multiplyVector(PP1);
        PP2 = m.multiplyVector(PP2);
        PP3 = m.multiplyVector(PP3);
        PNormal.set(0,0,-1);
        if(segmentPlaneIntersect(P0,P1,PNormal,PP0,PP1,PP2,PP3)){
            collisionDetected = true;
            cerr <<"!!!"<<endl;
            return;
        }

    }

}
