#include "../global.h"
using namespace std;

GLuint drawBranch(bool detailFlag,float color[])
{
//	cout << "drawBranches called!" << endl;
	int nVerts, nIndices, *indices;
	float *vertices, *normals, *texcoords;

	if(detailFlag){
//		cout << "ObjReader reading branchMore.obj!" << endl;
		ObjReader::readObj("tree/branchMore.obj",nVerts,&vertices,&normals,&texcoords,nIndices,&indices);
	}else{
//		cout << "ObjReader reading branchLess.obj!" << endl;
		ObjReader::readObj("tree/branchLess.obj",nVerts,&vertices,&normals,&texcoords,nIndices,&indices);
	}

//	cout << "Generating display list!" << endl;

	GLuint branch;
	branch = glGenLists(1);

	glNewList(branch, GL_COMPILE);
        glColor4f(color[0],color[1],color[2],color[3]);
		glBegin(GL_TRIANGLES);
		for(int i=0; i<nIndices; i++)
		{
			if(normals) glNormal3fv(normals+3*i);
			if(texcoords && detailFlag) glTexCoord2fv(texcoords+2*i);
			glVertex3fv(vertices+3*i);
		}
		glEnd();
		glColor3f(1,1,1);
	glEndList();

	delete []vertices;
	delete []normals;
	delete []texcoords;

//	cout << "drawBranches returning: " << branch << endl;
	return branch;
}

void fractal(int &angle, int level, int &threshLv, float diffuse[], float change[], float scale, unsigned int &branchMore, unsigned int &branchLess)
{
		// deciding branching number
	float branchNo=1;
	for(int i=0; i<(level<1?1:level); i++){
		branchNo*=(rand()%4+1)/4.0;}
	branchNo=(int)(branchNo*4+2);

		// deciding rotation angle
	for(int i=0; i<branchNo; i++){
		float zRot=1;
		zRot*=(rand()%(angle/(level/3+1))+1)/(float)angle;
		zRot*=angle;

		float yRot=rand()%360;

		glPushMatrix();
		glRotatef(zRot,0,0,1);
		glRotatef(yRot,0,1,0);
		glScalef(scale, scale, scale);
		glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
		if(level<=threshLv) glCallList(branchLess);
		else glCallList(branchMore);

		if(!level){
			glPopMatrix();}
		else{
			glTranslatef(0,9.7,0);
			if(level<=threshLv){
				float newDiffuse[4];
				for(int k=0; k<4; k++) newDiffuse[k]=diffuse[k]+change[k];
				fractal(angle,level-1,threshLv,newDiffuse,change,0.85,branchMore,branchLess);}
			else{
				fractal(angle,level-1,threshLv,diffuse,change,0.6,branchMore,branchLess);}
			glPopMatrix();
        }
    }
}

GLuint compileTree(int angle, int level, float *color)
{
//	cout << "compileTree called!" << endl;
	float diffuse[]={0.65,0.43,0.24,1};
	int threshLv=(level+1)/2;
	float change[]={-0.3/threshLv,0.35/threshLv,-0.04/threshLv,0};

	if(color){
		for(int i=0; i<4; i++){
			diffuse[i]=color[i];
			change[i]=(color[i+4]-color[i])/threshLv;
		}
	}
//	cout << "compiling branches!" << endl;
	GLuint branchMore=drawBranch(1,color);
	GLuint branchLess=drawBranch(0,color+4);

	int width, height;
	unsigned char *texImg=loadPPM("tree/treeTex.ppm",width,height);
	GLuint texName;

//	cout << "generating textures!" << endl;
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0,GL_RGB,GL_UNSIGNED_BYTE,texImg);
	delete []texImg;

	//srand(clock());

//	cout << "compiling tree!" << endl;
	GLuint tree=glGenLists(1);
	glNewList(tree,GL_COMPILE);
        glColor3f(color[0],color[1],color[2]);
        //glEnable(GL_NORMALIZE);
		glBindTexture(GL_TEXTURE_2D, texName);
		glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
		glCallList(branchMore);
		glTranslatef(0,9.7,0);
		fractal(angle, level-1, threshLv, diffuse, change, 0.55, branchMore, branchLess);
		glBindTexture(GL_TEXTURE_2D, 0);
		//glDisable(GL_NORMALIZE);
        //glColor3f(1,1,1);
	glEndList();

//	cout << "compileTree returning: " << tree << endl;
	return tree;
}
